import pytest

from nmicheck.tools import long_nmi, obfuscate_nmi

EXAMPLE_CHECKSUMS = [
    ("2001985732", 8),
    ("2001985733", 6),
    ("7102000001", 7),
    ("QAAAVZZZZZ", 3),
    ("QCDWW00010", 2),
    ("qcdww00010", 2),
    ("VKTS876510", 8),
]


EXAMPLE_HASHES = [
    ("2001985732", "6D34CC122D491FC199BC"),
    ("20019857328", "6D34CC122D491FC199BC"),
    ("QCDWW00010", "2B6672795F589E3AA0D2"),
    ("qcdww00010", "2B6672795F589E3AA0D2"),
    ("VKTS876510", "4BB52AD1D75AF68F0B91"),
]


@pytest.mark.parametrize("nmi,checksum", EXAMPLE_CHECKSUMS)
def test_long_nmi(nmi, checksum):
    """Test the checksum expansion of NMIs"""

    assert long_nmi(nmi) == f"{nmi}{checksum}".upper()


@pytest.mark.parametrize("nmi,hashed", EXAMPLE_HASHES)
def test_obfuscated_nmi(nmi, hashed):
    """Test the obfuscattion of NMIs"""
    assert obfuscate_nmi(nmi) == hashed
    assert obfuscate_nmi(nmi, salt="newsecret") != hashed



"""Functions to validate National Metering Identifiers (NMIs)"""

from .check import checksum_valid, nmi_checksum
from .tools import long_nmi, obfuscate_nmi, short_nmi
from .version import __version__

__all__ = [
    "__version__",
    "checksum_valid",
    "long_nmi",
    "nmi_checksum",
    "obfuscate_nmi",
    "short_nmi",
]



import math


def nmi_checksum(nmi: str) -> int:
    """Return the checksum digit of an NMI"""
    if len(nmi) != 10:
        msg = "NMI must be 10 digits"
        raise ValueError(msg)
    total = 0
    for i, char in enumerate(reversed(nmi)):
        val_ascii = ord(char)
        val = val_ascii if i % 2 == 1 else val_ascii * 2
        to_add = sum(map(int, str(val)))
        total += to_add

    next_ten = math.ceil(total / 10.0) * 10
    checksum = next_ten - total
    return checksum


def checksum_valid(nmi: str) -> bool:
    """Return whether the 11th checksum digit is valid"""
    if len(nmi) != 11:
        msg = "NMI must be 11 digits"
        raise ValueError(msg)
    start = nmi[0:10]
    try:
        checksum = int(nmi[10])
    except ValueError:
        return False  # Checksum must be an integer
    if checksum == nmi_checksum(start):
        return True
    return False



import pytest

from nmicheck import checksum_valid, nmi_checksum

EXAMPLE_CHECKSUMS = [
    ("2001985732", 8),
    ("2001985733", 6),
    ("7102000001", 7),
    ("QAAAVZZZZZ", 3),
    ("QCDWW00010", 2),
    ("qcdww00010", 2),
    ("VKTS876510", 8),
]


@pytest.mark.parametrize("nmi,checksum", EXAMPLE_CHECKSUMS)
def test_checksum(nmi, checksum):
    """Test the checksum calculation"""
    assert nmi_checksum(nmi) == checksum


@pytest.mark.parametrize("nmi,checksum", EXAMPLE_CHECKSUMS)
def test_validate(nmi, checksum):
    """Test the checksum calculation"""

    full_nmi = f"{nmi}{checksum}"
    assert checksum_valid(full_nmi)


def test_bad_checksum():
    """Test the checksum fails"""

    full_nmi = "QAAAVZZZZZX"
    assert not checksum_valid(full_nmi)



import hashlib

from .check import nmi_checksum


def short_nmi(nmi: str) -> str:
    """Truncate a NMI to 10 characters"""
    return nmi[:10].upper()


def long_nmi(nmi: str) -> str:
    """Expand a NMI to 11 characters"""
    short = short_nmi(nmi)
    return short + str(nmi_checksum(short))


def obfuscate_nmi(nmi: str, salt: str = "secret", length: int = 20) -> str:
    """Make a pseudo anonymous version of the NMI"""
    clean = short_nmi(nmi)
    salted = f"{salt}-{clean}"
    nmi_hash = hashlib.sha256(salted.encode("utf-8")).hexdigest().upper()
    return nmi_hash[:length]
